<?php

namespace Drupal\zeptomail\Plugin\Mail;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Mail\MailInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Component\Utility\Html;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\zeptomail\Helper\ZeptoMailApi;
use Drupal\Core\Mail\MailFormatHelper;
/**
 * Modify the Drupal mail system to use Zoho ZeptoMail when sending emails.
 *
 * @Mail(
 *   id = "zeptomail_mail",
 *   label = @Translation("ZeptoMail mailer"),
 *   description = @Translation("Sends the message using ZeptoMail.")
 * )
 */
class ZeptoMail implements MailInterface, ContainerFactoryPluginInterface {

  /**
   * Configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;
 
  public function __construct(ImmutableConfig $settings, LoggerInterface $logger) {
    $this->config = $settings;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $container->get('config.factory')->get('zeptomail.settings'),
      $container->get('logger.factory')->get('zeptomail')
    );
  }

  /**
   * Concatenate and wrap the e-mail body for either plain-text or HTML e-mails.
   *
   * @param array $message
   *   A message array, as described in hook_mail_alter().
   *
   * @return array
   *   The formatted $message.
   */
  public function format(array $message) {
    // Join the body array into one string.
    if (is_array($message['body'])) {
      $message['body'] = implode("\n\n", $message['body']);
    }

    // If text format is specified in settings, run the message through it.
    $content_type = $message['headers']['Content-Type'];
      
    if (strpos($content_type, 'text/plain') !== FALSE) {
      $message['body'] =  MailFormatHelper::htmlToText($message['body']);
    }

    return $message;
  }

 
  public function mail(array $message) {
    $current_user = \Drupal::currentUser();
    $domain = $this->config->get('domain');
		$mailtoken = base64_decode($this->config->get('mailtoken'));
    $zeptoAPI = new ZeptoMailApi($domain,$mailtoken);
    $headers = $message['headers'];

    $mail_data = [
      'subject' => $message['subject']
    ];
    $fromAddress =  [
			'name' => $this->config->get('from_name'),
			'address' => $this->config->get('from_address')
			];
      $mail_data['from'] =  $fromAddress;
    if ($message['to'] !== null) {
        $toRecipients = $message['to'];
        $mail_data['to'] = $this::getEmailDetails($message['to']);
    }

    if (!empty($message['headers']['Content-Type'])) {
      $content_type = $message['headers']['Content-Type'];
      if (strpos($content_type, 'text/plain') !== FALSE) {
        $mail_data['textbody'] = $message['body'];
      } 
      else {
        $mail_data['htmlbody'] = $message['body'];
      }
    } else {
      $mail_data['htmlbody'] = $message['body'];
    }
    
    // Add Cc / Bcc headers.
    if (!empty($message['Cc'])) {
      $mail_data['cc'] = $message['Cc'];
    }
    if (!empty($message['Bcc'])) {
      $mail_data['bcc'] = $message['Bcc'];
    }



    // Make sure the files provided in the attachments array exist.
    if (!empty($message['params']['attachments'])) {
      $attachmentJSONArr = array();
      $count = 0;
      $filesystem = \Drupal::service('file_system');
      foreach ($message['params']['attachments'] as $attachment) {
        $isFile = false;
        if (!empty($attachment['filepath']) && file_exists($attachment['filepath'])) {
          $fileContent = file_get_contents($attachment['filepath']);
          $attach = [
            'content' => base64_encode($fileContent),
            'name' => basename($attachment['filepath']),
            'mime_type' => mime_content_type($fileContent)
          ];
          $isFile =true;
        }
        elseif (!empty($attachment['filecontent']) && !empty($attachment['filename'])) {
          $attach = [
            'content' => base64_encode($attachment['filecontent']),
            'name' => $attachment['filename'],
            'mime_type' =>  $attachment['filemime']
          ];
          
          $isFile =true;
        }
        if($isFile) {
          $attachmentJSONArr[$count] = $attach;
          $count = $count+1;
        }
      }
 

      if (count($attachmentJSONArr) > 0) {
        
       $mail_data['attachments'] = $attachmentJSONArr;
      } 
    }

$responseObj = $zeptoAPI->sendZeptoMail($mail_data);

if($responseObj->result !== "success") {
  $this->logger->error(json_encode($responseObj));
  return false;
}
return true;
    
  }

  private function splitNameAndAddress($emailAddress) {
		$result = [
			'name' => '',
			'address' => ''
			];
		$pos = strpos($emailAddress, '<');
		if($pos !== false) {
			$result['address'] =  substr($emailAddress, $pos+1, strlen($emailAddress)-$pos-2);
			$result['name'] = substr($emailAddress,0,$pos-1);
		} else {
			$result['address'] = $emailAddress;
		}
		
		return $result;
	}
	private function getEmailDetails($emailAddressDetails) {
		if(!is_array($emailAddressDetails)){
			$emailAddressDetails = explode( ',', $emailAddressDetails );
		}
		$result = array();
		
		
		foreach($emailAddressDetails as $emailAddress){
			$emailDetail = array();
			$emailDetail["email_address"] = $this::splitNameAndAddress($emailAddress);
			array_push($result,$emailDetail);
		}
		
		return $result;
	}

}
